@@ -1,16 +1,19 @@
-#' Model Performance Plots
+#' Calculate Model Performance
 #'
-#' @param explainer a model to be explained, preprocessed by the 'explain' function
+#' Prepare a data frame with model residuals.
+#'
+#' @param explainer a model to be explained, preprocessed by the \code{\link{explain}} function
 #' @param ... other parameters
 #'
-#' @return An object of the class 'model_performance_explainer'.
+#' @return An object of the class \code{model_performance_explainer}.
 #' @references Predictive Models: Visual Exploration, Explanation and Debugging \url{https://pbiecek.github.io/PM_VEE/}
 #' @export
 #' @examples
 #'  \dontrun{
 #' library("randomForest")
-#' HR_rf_model <- randomForest(status == "fired"~., data = HR, ntree = 100)
+#' HR_rf_model <- randomForest(as.factor(status == "fired")~., data = HR, ntree = 100)
 #' explainer_rf  <- explain(HR_rf_model, data = HR, y = HR$status == "fired")
+#' # resulting dataframe has predicted values and residuals
 #' model_performance(explainer_rf)
 #'
 #' HR_glm_model <- glm(status == "fired"~., data = HR, family = "binomial")
@@ -29,10 +32,22 @@ model_performance <- function(explainer, ...) {
   if (!("explainer" %in% class(explainer))) stop("The model_performance() function requires an object created with explain() function.")
   if (is.null(explainer$data)) stop("The model_performance() function requires explainers created with specified 'data' parameter.")
   if (is.null(explainer$y)) stop("The model_performance() function requires explainers created with specified 'y' parameter.")
-
+  # Check since explain could have been run with precalculate = FALSE
+  if (is.null(explainer$y_hat)){
+    predicted <- explainer$predict_function(explainer$model, explainer$data, ...)
+  } else {
+    predicted <- explainer$y_hat
+  }
   observed <- explainer$y
-  predicted <- explainer$predict_function(explainer$model, explainer$data, ...)
-  residuals <- data.frame(predicted, observed, diff = predicted - observed)
+  # Check since explain could have been run with precalculate = FALSE
+  if (is.null(explainer$residuals)){
+    diff <- predicted - observed
+  } else {
+    # Negative to save constistency. model_performance used to use `predicted - observed` when explain precalculates `observed - predicted`
+    diff <- -explainer$residuals
+  }
+
+  residuals <- data.frame(predicted, observed, diff = diff)
 
   class(residuals) <- c("model_performance_explainer", "data.frame")
   residuals$label <- explainer$label
@@ -40,7 +55,7 @@ model_performance <- function(explainer, ...) {
 }
 
 
-#' Model Performance Summary
+#' Print Model Performance Summary
 #'
 #' @param x a model to be explained, object of the class 'model_performance_explainer'
 #' @param ... other parameters
