@@ -11,6 +11,9 @@ model_classif_glm <- glm(status == "fired"~., data = HR, family = "binomial")
 model_classif_rf <- randomForest::randomForest(status == "fired"~., data = HR, ntree = 50)
 model_regr_rf <- randomForest::randomForest(m2.price~., data = apartments, ntree = 50)
 model_regr_lm <- lm(m2.price~., data = apartments)
+library("gbm")
+gbm1 <- gbm(m2.price ~ ., data = apartments, distribution = "gaussian")
+gbm2 <- gbm(survived~., data = titanic_imputed, distribution = "bernoulli")
 
 # explain()
 p_fun_rf <- function(model, x) predict(model, x)
